/*!
 * \file asuro.h
 * \brief Definitionen und Funktionen der ASURO Bibliothek
 *
 * $Revision: 2.70 $
 * $Date: 07. Januar 2007 $
 * $Author: Jan Grewe, Robotrixer, Waste, Stochri, Andun, m.a.r.v.i.n $
 *

  \version  V001 - 10.02.2007 - m.a.r.v.i.n\n
            +++ my_t Neue Datenstruktur\n
            Datenstruktur fuer die Asuro-Hardwareabhaengigen Parameter die
            jeder User fuer seinen Asuro in der Datei myasuro.h selber
            einstellen kann um die Asuro-Typischen Eigenschaften zu definieren.
  \version  V002 - 18.02.2007 - Sternthaler\n
            +++ my_t\n
            - Neue Variable fuer MY_MOTOR_DIFF zum ausgleichen unterschiedlicher\n
              Motoren.
            - Aendern aller enthaltenen Variablen auf einen moeglichst kleinen\n
              Speicherbedarf. Notwendige Typumwandlungen muessen im Code durch\n
              casten erfolgen.
            V003 - 20.02.2007 - m.a.r.v.i.n\n  
            +++ my_t Datenstruktur wieder entfernt\n
            Es werden direkt die Defines aus der myasuro.h verwendet.
*/
/***************************************************************************
*
*it is not allowed to remove the nicknames of the contributers to this software
* from the function header
*/
/***************************************************************************
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   any later version.                                                    *
***************************************************************************/

/****************************************************************************
*
* File Name:   asuro.c
*
*
* Ver.     Date         Author           Comments
* -------  ----------   --------------   ------------------------------
* 1.00     14.08.2003   Jan Grewe    build
* 2.00     14.10.2003   Jan Grewe        RIGHT_VEL -> MotorSpeed(unsigned char left_speed, unsigned char right_speed);
*                                        LeftRwd(),LeftFwd(),RightRwd(), LEFT_VEL,
*                                        RigthFwd() -> MotorDir(unsigned char left_dir, unsigned char right_dir);
*                                        GREEN_ON,GREEN_OFF,RED_ON,RED_OFF -> StatusLED(unsigned char color);
*                                        LED_RED_ON, LED_RED_OFF -> FrontLED(unsigned char color);
*                                        Blink(unsigned char left, unsigned char right) ->
*                                         BackLED(unsigned char left, unsigned char right);
*                                        Alles in Funktionen gefasst => leichter verstndlich ?!?!
* 2.10     17.10.2003   Jan Grewe        new Timer funktion void Sleep(unsigned char time36kHz)
*
*
* Copyright (c) 2003 DLR Robotics & Mechatronics
*****************************************************************************/
/****************************************************************************
*
* File Name:   asuro.c
* Project  :   asuro library "Robotrixer Buxtehude"
*
* Description: This file contains additional functions:
*
* signal (SIG_ADC)                 interrupt/signal routine for encoder-counter
* signal (SIG_INTERRUPT1)          signal for switches
* EncoderInit()                    initializing encoder-counter
* EncoderStart()                   start autoencoding
* EncoderStop()                    stop autoencoding
* EncoderSet(int,int)              set encodervalue
* Msleep(int delay)                wait for delay in milliseconds
* Gettime()                        get systemtime in milliseconds
* PrintInt(int)
*
* modifications in Sleep, SIG_OUTPUT_COMPARE2, PollSwitch, LineData
*
* Ver.     Date         Author           Comments
* -------  ----------   --------------   ------------------------------
* beta1    31.03.2005   Robotrixer   asuro library
* -------  ----------   --------------   ------------------------------
* the encoder source is based on RechteckDemo.c ver 2.0 by Jan Grewe 22.10.2003
* Copyright (c) 2003 DLR Robotics & Mechatronics

*****************************************************************************/
/****************************************************************************
*
* File Name:   asuro.c
* Project  :   asuro library modified for IR collision detector
*
* Description: modifications made in following functions:
*
* SIGNAL (SIG_OUTPUT_COMPARE2)  ->  SIGNAL (SIG_OVERFLOW2)
* Gettime()       counts now 36kHz
* Init()        timer2 modified for adjustable duty cycle
* Batterie()        bug fixed
* Sleep()       counts now 36kHz
* Msleep()        counts now 36kHz
*
* Ver.     Date         Author           Comments
* -------  ----------   --------------   ------------------------------
* beta2    11.06.2005   Waste      asuro library
* -------  ----------   --------------   ------------------------------
*****************************************************************************/
/****************************************************************************
*
* File Name:   asuro.c
* Project  :   asuro library
*
* Description: This file contains additional functions:
*
* motor control functions 29.7.2005 stochri
* void Go(int distance)
* void Turn(int degree)
*
* unsigned char Wheelspeed[2]   measured Wheelspeed by interupt
*
* Ver.     Date         Author           Comments
* -------  ----------   --------------   ------------------------------------------
* sto1     29.07.2005   stochri    asuro library with motor control functions
* -------  ----------   --------------   ------------------------------------------
*****************************************************************************/
/****************************************************************************
*
* File Name:   asuro.c
* Project  :   asuro library "Robotrixer Buxtehude"
*
* Description: modifications made in following function:
*
* Go (int distance, int speed)           Added Speed and Odometrie
* Turn (int degree, int speed)           Added Speed and Odometrie
*
* modifications in Sleep, SIG_OUTPUT_COMPARE2, PollSwitch, LineData
*
* Ver.     Date         Author           Comments
* -------  ----------   --------------   ------------------------------
* And1     31.07.2005   Andun    See above
* -------  ----------   --------------   ------------------------------
*
*****************************************************************************/
/****************************************************************************
*
* File Name:   asuro.c
* Project  :   asuro library
*
* Description: modifications made in following functions:
*
* void PrintInt(int wert)
*
* unsigned char Wheelspeed[2]    removed
*
* Ver.     Date         Author           Comments
* -------  ----------   --------------   ------------------------------------------
* 2.60     28.09.2005   m.a.r.v.i.n      doxygen comments
* -------  ----------   --------------   ------------------------------------------
*****************************************************************************/
/****************************************************************************
*
* File Name:   asuro.c
* Project  :   asuro library
*
* Description: modifications made in following functions:
*
* SIGNAL (SIG_ADC)
* void PrintInt(int wert)
*
*
* Ver.     Date         Author           Comments
* -------  ----------   --------------   ------------------------------------------
* 2.61     20.11.2006   m.a.r.v.i.n      SIGNAL (SIG_ADC): static Variable toggle initialisiert
*                                        auf False (Bug report von Rolf_Ebert)
*                                        PrintInt: Initialisierung text String kann zu Fehler
*                                        beim Flashen mit RS232/IR Adapter fuehren
*                                        (Bug report von francesco)
* -------  ----------   --------------   ------------------------------------------
*****************************************************************************/
/****************************************************************************
*
* File Name:   asuro.c
* Project  :   asuro library
*
* Description: new functions has been added:
*
* void uart_putc(unsigned char zeichen)
* void SerPrint(unsigned char *data)
* void SetMotorPower(int8_t left_speed, int8_t right_speed )
* void sound(uint16_t freq, uint16_t duration_msec, uint8_t amplitude)
*
* Description: modifications made in following functions:
*
* void Go(int distance, int power)
* void Turn(int degree, int speed)
* void PrintInt(int wert)
*
*
* Ver.     Date         Author           Comments
* -------  ----------   --------------   ------------------------------------------
* 2.70     07.01.2007   stochri          new functions:
*                                        uart_putc: send single character
*                                        SerPrint: send 0-terminated string
*                                        SetMotorPower: set Motor speed and direction
*                                        sound: Sound Ausgabe ueber Motor PWM
*                                        Go: distance in mm
*                                        Turn: comments
*                       m.a.r.v.i.n      PrintInt: SerWrite ersetzt durch SerPrint
* -------  ----------   --------------   ------------------------------------------
*****************************************************************************/


#ifndef ASURO_H
#define ASURO_H

#include <avr/io.h>
#include <avr/interrupt.h>
#ifndef SIGNAL
  #include <avr/signal.h>   // header file obsolete in actual avr-libc
  #include <inttypes.h>
#endif
#include <stdlib.h>

#define  FALSE  0
#define  TRUE 1

#define  OFF    0
#define  ON     1

#define GREEN 1
#define RED 2
#define YELLOW  3

/* neue Funktionen und Variablen*/
#define LEFT    0
#define RIGHT   1
#define CENTER  2

/* Definitionen fr I2C */
#define READ    1
#define WRITE   0
#define ACK     1
#define NAK     0

/* --- Globale Variablen -----------------------------------*/
/*!
 * Asuro Lib Versions String
 */
extern const char version[5];

/*
 * Tastsensor Wert bei Interrupt Betrieb. 0=keine Taste, 1= Taste gedrueckt
 */
extern int switched;

/*
 * Odometriesensor Zaehler bei Interrupt Betrieb.
 * encoder[0] links, encoder[1] = rechts.
 */
extern int encoder[2];

/*
 * Counter fuer 36kHz.
 * Wird in der Interrupt Funktion SIG_OVERFLOW2 hochgezaehlt\n
 * und in der Sleep() Funktion abgefragt.
 */
extern volatile unsigned char count36kHz;

/*
 * Sytemzeit in ms.
 * Wird in der Interrupt Funktion SIG_OVERFLOW2 hochgezaehlt
 * und in der Gettime() Funktion verwendet.
 */
extern volatile unsigned long timebase;

/*
 * Odometrie Sensor Abfrage im Interrupt Betrieb.
 * Wird in der Interrupt Funktion SIG_ADC abgefragt,
 * in der EncoderInit() und EncoderStart() Funktion gesetzt
 * und in der EncoderStop() Funktion geloescht.
 */
extern volatile int autoencode;

/* --- Funktions Prototypen -----------------------------------*/

// motor control functions 29.7.2005 stochri
// motor control functions 31.7.2005 edited by Andun

void Init(void);

unsigned long Gettime(void);
void Msleep(int dauer);
void Sleep(unsigned char time36kHz);

void EncoderInit(void);
void EncoderSet(int setl,int setr);
void EncoderStop(void);
void EncoderStart(void);
void Go(int distance, int speed);
void Turn(int degree, int speed);

void UartPutc(unsigned char zeichen);
void SerPrint(unsigned char *data);
void PrintInt(int wert);
void PrintLong(long wert);

int Batterie(void);
void LineData(unsigned int *data);
void OdometrieData(unsigned int *data);

inline void StatusLED(unsigned char color);
inline void FrontLED(unsigned char status);
void BackLED(unsigned char left, unsigned char right);

inline void MotorDir(unsigned char left_dir, unsigned char right_dir);
inline void MotorSpeed(unsigned char left_speed, unsigned char right_speed);
void SetMotorPower(int8_t leftpwm, int8_t rightpwm);

void SerWrite(unsigned char *data,unsigned char length);
void SerRead(unsigned char *data, unsigned char length, unsigned int timeout);

unsigned char PollSwitch (void);
void StartSwitch(void);
void StopSwitch(void);

void Sound(uint16_t freq, uint16_t duration_msec, uint8_t amplitude);

/* ----------- END ------------ */


/* --------------- INTERNAL ------------- */
#define GREEN_LED_ON  PORTB |=  GREEN_LED /*!< Gruene Status LED an */
#define GREEN_LED_OFF PORTB &= ~GREEN_LED /*!< Gruene Status LED aus */
#define RED_LED_ON    PORTD |=  RED_LED   /*!< Rote Status LED an */
#define RED_LED_OFF   PORTD &= ~RED_LED   /*!< Rote Status LED aus */

#define FWD (1 << PB5)                    /*!< Motor vorwaerts */
#define RWD (1 << PB4)                    /*!< Motor rueckwaerts */
#define BREAK 0x00                        /*!< Motor bremsen */
#define FREE  (1 << PB4) | (1 << PB5)     /*!< Motor freilaufend */

#define IRTX        (1 << PB3)            /*!< PB3 Port fuer Infrarot Transmitter LED */
#define GREEN_LED   (1 << PB0)            /*!< PB0 Port fuer Gruene Status LED */
#define RED_LED     (1 << PD2)            /*!< PD2 Port fuer Rote Status LED */

#define PWM       (1 << PB1) | (1 << PB2) /*!< PB1, PB2 Ports fuer Pulsweitenmodulation der Motor Geschwindigkeit */
#define RIGHT_DIR (1 << PB4) | (1 << PB5) /*!< PB4, PB5 Ports fuer Drehrichtung rechter Motor */
#define LEFT_DIR  (1 << PD4) | (1 << PD5) /*!< PD4, PD5 Ports fuer Drehrichtung linker Motor */

#define SWITCHES   (1 << PD3)             /* PD3 Port fuer Tastsensor */
#define SWITCH_ON  PORTD |= SWITCHES      /* Tastsensor an  */
#define SWITCH_OFF PORTD &= ~SWITCHES     /* Tastsensor aus */

#define BATTERIE  (1 << MUX0) | (1 << MUX2)   /*!< ADC5 A/D Wandler Port fuer Batterie Abfrage */
#define SWITCH    (1 << MUX2)                 /*!< ADC4 A/D Wandler Port fuer Tastsensor */
#define IR_LEFT   (1 << MUX0) | (1 << MUX1)   /*!< ADC3 A/D Wandler Port fuer Linienfolger Fototransistor links */
#define IR_RIGHT  (1 << MUX1)                 /*!< ADC2  A/D Wandler Port fuer Linienfolger Fototransistor rechts */
#define FRONT_LED (1 << PD6)                  /*!< PD6 Port fuer Front LED */

#define ODOMETRIE_LED     (1 << PD7)              /*!< PD7 Port fuer  Odometrie LED */
#define ODOMETRIE_LED_ON  PORTD |= ODOMETRIE_LED  /*!< Odometrie LED an  */
#define ODOMETRIE_LED_OFF PORTD &= ~ODOMETRIE_LED /*!< Odometrie LED aus */

#define WHEEL_LEFT   (1 << MUX0)              /*!< ADC1 A/D Wandler Port fuer Odometrie Sensor links*/
#define WHEEL_RIGHT  0                        /*!< ADC0 A/D Wandler Port fuer Odometrie Sensor rechts */

#endif /* ASURO_H */
